@@ -40,6 +40,8 @@ our @EXPORT    = qw(
 										 fetch_conf
 										 get_counter
 										 exists_counter
+										 dump_config
+parse_conf_fn
 										 %CONF
 										 $DIMS
 									);
@@ -69,75 +71,32 @@ use Circos::Error;
 our %CONF;
 our $DIMS;
 
-# -------------------------------------------------------------------
-sub get_counter {
-	my $counter = shift;
-	confess if ! defined $counter;
-	if (! exists_counter($counter)) {
-		fatal_error("configuration","no_counter",$counter);
-	} else {
-		return $CONF{counter}{$counter};
-	}
-}
-
-# -------------------------------------------------------------------
-sub exists_counter {
-	my $counter = shift;
-	return defined $CONF{counter}{$counter};
-}
-
-# -------------------------------------------------------------------
-sub increment_counter {
-  my ($counter,$value) = @_;
-  init_counter($counter,0);
-	if (defined $value) {
-		$CONF{counter}{$counter} += $value;
-		printdebug_group("counter","incrementing counter",$counter,$value,"now",get_counter($counter));
-	}
-}
-
-sub set_counter {
-  my ($counter,$value) = @_;
-	if (defined $value) {
-		$CONF{counter}{$counter} = $value;
-		printdebug_group("counter","set counter",$counter,$value,"now",get_counter($counter));
-	}
-}
-
-{
-	my %seen;
-	sub init_counter {
-		my ($counter,$value) = @_;
-		if (! $seen{$counter}++) {
-			if (defined $value) {
-				$CONF{counter}{$counter} = $value;
-			}
-		}
-		printdebug_group("counter","init counter",$counter,"with value",$value,"new value",get_counter($counter));
-	}
-}
+our $COUNTER;
 
 # -------------------------------------------------------------------
 # Return the configuration hash leaf for a parameter path.
 #
-# fetch_configuration("ideogram","spacing")
+# fetch_configuration("ideogram","spacing") -> $CONF{ideogram}{spacing}
 #
-# returns
+# alias: fetch_conf("ideogramn","spacing")
 #
-# $CONF{ideogram}{spacing}
+# fetch_configuration() -> $CONF
 #
 # If the leaf, or any of its parents, do not exist, undef is returned.
 sub fetch_configuration {
-	my @config_path = @_;
-	my $conf_item   = \%CONF;
-	for my $path_element (@config_path) {
-		if (! exists $conf_item->{$path_element}) {
+    my @config_path = @_;
+    my $node        = \%CONF;
+    if(! @config_path) {
+	return \%CONF;
+    }
+    for my $path_element (@config_path) {
+	if (! exists $node->{$path_element}) {
 	    return undef;
-		} else {
-	    $conf_item = $conf_item->{$path_element};
-		}
+	} else {
+	    $node = $node->{$path_element};
 	}
-	return $conf_item;
+    }
+    return $node;
 }
 
 sub fetch_conf {
@@ -145,9 +104,6 @@ sub fetch_conf {
 }
 
 # -------------------------------------------------------------------
-# 
-#
-#
 #
 sub fetch_parameter_list_item {
 	my ($list,$item,$delim) = @_;
@@ -155,6 +111,37 @@ sub fetch_parameter_list_item {
 	return $parameter_hash->{$item};
 }
 
+sub dump_config {
+	my %OPT = @_;
+	$Data::Dumper::Indent    = 2;
+	$Data::Dumper::Quotekeys = 0;
+	$Data::Dumper::Terse     = 0;
+	$Data::Dumper::Sortkeys  = 1;
+	$Data::Dumper::Varname   = "CONF";
+	if ($OPT{cdump}) {
+		my ($path,$rx) = split(":",$OPT{cdump});
+		my @path       = split(/[.\/]/,$path);
+		if($rx) {
+			$Data::Dumper::Varname .= join(":",join("_",@path),$rx);
+			$Data::Dumper::Sortkeys = sub { 
+				my ($h) = @_;
+				return [sort grep($_ =~ /$rx/, keys %$h)];
+			}
+		} else {
+			$Data::Dumper::Varname .= join("_",@path);
+		}
+		if(@path) {
+			printdumper(get_hash_leaf(\%CONF,@path));
+		} else {
+			printdumper(\%CONF);
+		}
+	} else {
+
+		printdumper(\%CONF);
+	}
+	exit
+}
+
 # -------------------------------------------------------------------
 # Given a string that contains a list, like
 #
@@ -223,17 +210,62 @@ sub parse_var_value {
 sub populateconfiguration {
 
   my %OPT = @_;
-	
+
   for my $key ( keys %OPT ) {
 		if (defined $OPT{$key}) {
 			if ($key eq "debug_group") {
-	      $CONF{$key} .= ",".$OPT{$key};
+				my @new_groups      = split(",",$OPT{$key});
+				my @existing_groups = split(",",$CONF{$key});
+				my $reset = 0;
+				for my $new_group (@new_groups) {
+					if($new_group =~ /^([-+])(.+)/) {
+						my ($flag,$new_group_name) = ($1,$2);
+						if($flag eq "-") {
+							@existing_groups = grep($_ ne $new_group_name, @existing_groups);
+						} elsif ($flag eq "+") {
+							push @existing_groups, $new_group_name;
+						}
+					} else {
+						@existing_groups = () if ! $reset++;
+						push @existing_groups, $new_group;
+					}
+				}
+				@existing_groups = grep(defined $_, @existing_groups);
+	      $CONF{$key} = join($COMMA,sort(uniq(@existing_groups))) if uniq(@existing_groups);
+			} elsif ($key eq "param") {
+				for my $pair (@{$OPT{$key}}) {
+					my ($param,$value) = split(/=/,$pair);
+					$value = 0 if lc $value eq "no"  || lc $value eq "n";
+					$value = 1 if lc $value eq "yes" || lc $value eq "y";
+					my @param_path = split("/",$param);
+					my $leaf = \%CONF;
+					for my $path (@param_path[0..@param_path-2]) {
+						if(exists $leaf->{$path}) {
+							if(ref $leaf->{$path} eq "HASH") {
+								$leaf = $leaf->{$path};
+							} elsif (ref $leaf->{$path} eq "ARRAY") {
+								$leaf = $leaf->{$path};
+							} else {
+								fatal_error("configuration","parampath_missing",$param,$value,$path);
+							}
+						} else {
+							fatal_error("configuration","parampath_nothash",$param,$value,$path);
+						}
+					}
+					if(ref $leaf eq "HASH") {
+						$leaf->{ $param_path[-1] } = $value;
+					} elsif (ref $leaf eq "ARRAY") {
+						for my $l (@$leaf) {
+							$l->{ $param_path[-1] } = $value;
+						}
+					}
+				}
 			} else {
 	      $CONF{$key} = $OPT{$key};
 			}
 		}
   }
-
+	
 	# Combine top level hashes. This allows overriding values in blocks
 	# that are included.
 	#
@@ -241,10 +273,11 @@ sub populateconfiguration {
 	# <ideogram>
 	# label_size* = 12
 	# </ideogram>
-	
+
 	merge_top_hashes( \%CONF );
-	
-	# Fields like conf(key1,key2,...) are replaced by $CONF{key1}{key2}
+  resolve_synonyms( \%CONF, [] );
+
+  # Fields like conf(key1,key2,...) are replaced by $CONF{key1}{key2}
   #
   # If you want to perform arithmetic, you'll need to use eval()
   #
@@ -253,22 +286,21 @@ sub populateconfiguration {
   # The configuration can therefore depend on itself. For example, if
   #
   # flag = 10
-  #
   # note = eval(2*conf(flag))
+  repopulateconfiguration( \%CONF, undef, undef, undef, 1 );
+  repopulateconfiguration( \%CONF, undef, undef, undef, 0 );
 
-  resolve_synonyms( \%CONF, [] );
-  repopulateconfiguration( \%CONF );
   override_values( \%CONF );
   check_multivalues( \%CONF, undef );
 
-	fatal_error("configuration","no_housekeeping") if ! $CONF{housekeeping};
-
-  # populate some defaults
-  $CONF{'anglestep'}    ||= 1;
-  $CONF{'minslicestep'} ||= 5;
+  postprocess_values(\%CONF);
 
+  fatal_error("configuration","no_housekeeping") if ! $CONF{housekeeping};
+  
 }
 
+#-------------------------------------------------------------------
+# Iteratively run apply_synonym() on all configuration tree nodes.
 sub resolve_synonyms {
 	my $root = shift;
 	my $tree = shift;
@@ -294,13 +326,14 @@ sub resolve_synonyms {
 	}
 }
 
+#-------------------------------------------------------------------
+# Some configuration items may have more than one addressable name. 
 sub apply_synonym {
 	my ($value,$key,$tree) = @_;
 	my @synonyms = (
 									{
 									 key_rx => ".*::label_tangential", new_key => "label_parallel", action => "copy" },
 								 );
-    
 	my $key_name  = join(":",@$tree)."::".$key;
 	my ($new_key,$action);
 	for my $s (@synonyms) {
@@ -316,8 +349,7 @@ sub apply_synonym {
 }
 
 # -------------------------------------------------------------------
-# Parameters with *, **, ***, etc suffixes override those with
-# fewer "*";
+# Parameters with *, **, ***, etc suffixes override those with fewer "*";
 sub override_values {
 	my $root = shift;
 	my $parameter_missing_ok = 1;
@@ -325,6 +357,7 @@ sub override_values {
 		my @keys = keys %$root;
 		# do we have any parameters to override?
 		my @lengths = uniq ( map { length($_) } ( map { $_ =~ /([*]+)$/ } @keys ) );
+		my @delete_queue;
 		for my $len (sort {$a <=> $b} @lengths) {
 	    for my $key (@keys) {
 				my $rx = qr/^(.+)[*]{$len}$/;
@@ -335,16 +368,21 @@ sub override_values {
 					if ($parameter_missing_ok || grep($_ eq $key_name, @keys)) {
 						#printinfo("overriding",$key_name,$root->{$key_name},$key,$root->{$key});
 						$root->{$key_name} = $root->{$key};
+						# delete old key suffixed with *
+						push @delete_queue, $key;
 					}
 				}
 	    }
 		}
+		map { delete $root->{$_} } @delete_queue;
 		for my $key (keys %$root) {
-			my $value = $root->{$key};
-	    if (ref $value eq "HASH" ) {
-				#printinfo("iter",$key);
-				override_values($value);
-	    }
+			my @values = make_list($root->{$key});
+			for my $value (@values) {
+				if (ref $value eq "HASH" ) {
+					#printinfo("iter",$key);
+					override_values($value);
+				}
+			}
 		}
 	}
 }
@@ -371,14 +409,18 @@ sub merge_top_hashes {
 						condition  => { pass => qr/rule/ },
 						radius     => { pass => qr/tick/ },
 						axis       => { pass => qr/axes/ },
+            param      => { pass => qr/_root/},
+						post_increment_counter => { pass => qr/plot/ },
 					 };
-	
+
 	sub check_multivalues {
 		my ($root,$root_name,$level) = @_;
-		
+
 		return unless ref $root eq "HASH";
 		$level ||= 0;
-		
+
+		$root_name = "_root" if ! defined $root_name;
+
 		my @keys = keys %$root;
 		for my $key (@keys) {
 	    my $value = $root->{$key};
@@ -396,14 +438,13 @@ sub merge_top_hashes {
 				}
 				if (! $pass) {
 					printdumper($root);
-					fatal_error("configuration","multivalue",$key);
+					fatal_error("configuration","multivalue",$key,$root_name);
 				}
 			} elsif (ref $value eq "HASH") {
 				# this is a block
 	    }
 			#printinfo($level,$key,$value);
 		}
-		
 		for my $key (keys %$root) {
 	    my $value = $root->{$key};
 	    if (ref $value eq "HASH" ) {
@@ -415,18 +456,81 @@ sub merge_top_hashes {
 	}
 }
 
+# -------------------------------------------------------------------
+# Generic configuration iterator
+sub postprocess_values {
+	my $node = shift;
+	if (ref $node eq "HASH") {
+		for my $key (keys %$node) {
+	    my $value = $node->{$key};
+	    if (ref $value eq "HASH" ) {
+				#printinfo("conf_iterate","hash",$key);
+				postprocess_values($value);
+	    } elsif (ref $value eq "ARRAY") {
+				#printinfo("conf_iterate","list",$key);
+				map { postprocess_values($_) } @$value;
+	    } else {
+				#printinfo("conf_iterate","value",$key,$value);
+				if($value =~ /undef/i) {
+					#delete $node->{$key};
+					$node->{$key} = undef;
+				}
+			}
+		}
+	}
+}
+
+# -------------------------------------------------------------------
+# Generic configuration iterator
+sub conf_iterate {
+	my $node = shift;
+	if (ref $node eq "HASH") {
+		for my $key (keys %$node) {
+	    my $value = $node->{$key};
+	    if (ref $value eq "HASH" ) {
+				#printinfo("conf_iterate","hash",$key);
+				conf_iterate($value);
+	    } elsif (ref $value eq "ARRAY") {
+				#printinfo("conf_iterate","list",$key);
+				map { conf_iterate($_) } @$value;
+	    } else {
+				#printinfo("conf_iterate","value",$key,$value);
+				# do something with the value
+			}
+		}
+	}
+}
+
 sub set_counters {
-	my ($node,@paramfn) = @_;
+	my ($node,$parent_node_name,$parent_node,@paramfn) = @_;
 	my %set_counter_names;
 	for my $paramfn (@paramfn) {
 		my ($param,$fn) = @{$paramfn}{qw(param fn)};
-		if (my $value = $node->{$param}) {
-	    for my $counter_txt (split(",",$value)) {
-				my ($counter,$incr) = split(":",$counter_txt);
+		next if ! defined $node->{$param};
+		my @values      = make_list($node->{$param});
+		for my $value (grep(defined $_,@values)) {
+			for my $counter_txt (split(",",$value)) {
+				my ($counter,$incr);
+				if($counter_txt =~ /:/) {
+					($counter,$incr) = split(":",$counter_txt);
+				} else {
+					$counter = $parent_node_name;
+					$incr    = $counter_txt;
+				}
+				#printinfo($node,$parent_node_name,$counter,$incr);
 				$fn->($counter,$incr);
 				$set_counter_names{$counter}++;
-	    }
-		}	
+
+				# also increment counter of the node's type
+
+				if($parent_node_name eq "plot" && 
+					 ($node->{type} || ($parent_node && $parent_node->{type}))) {
+					my $counter_name = $node->{type} || $parent_node->{type};
+					$fn->( $counter_name, $incr );
+					$set_counter_names{ $counter_name }++;
+				}
+			}
+		}
 	}
 	return %set_counter_names;
 }
@@ -458,109 +562,212 @@ sub clone_merge {
 
 # -------------------------------------------------------------------
 sub repopulateconfiguration {
-  my ($node,$parent_node_name) = @_;
-	
-  my %set_counter_names = set_counters(
+  my ($node,$parent_node_name,$parent_node,$curr,$do_counter) = @_;
+
+  # compile current view of configuration paramters, respecting hierarchy
+  $curr ||= {};
+  $curr = { %$curr, %$node };
+
+	my %set_counter_names = set_counters(
 																			 $node,
-																			 {
-																				param=>"init_counter",				   fn=>\&init_counter 	   },
-																			 {
-																				param=>"pre_increment_counter", fn=>\&increment_counter },
-																			 {
-																				param=>"pre_set_counter",       fn=>\&set_counter       },
-																			);
-	
-  # default initializer, if init_counter was not called
-	if (ref $node && defined $parent_node_name) {
-		init_counter($parent_node_name,0) if ! $set_counter_names{$parent_node_name};
+																			 $parent_node_name,
+																			 $parent_node,
+																			 { param=>"init_counter",				   fn=>\&init_counter 	   },
+																			 { param=>"pre_increment_counter", fn=>\&increment_counter },
+																			 { param=>"pre_set_counter",       fn=>\&set_counter       },
+																			) if $do_counter;
+
+	if($do_counter) {
+		# default initializer, if init_counter was not called
+		if (ref $node && defined $parent_node_name) {
+			init_counter($parent_node_name,0) if ! $set_counter_names{$parent_node_name};
+			if($parent_node_name eq "plot" &&
+				 ($node->{type} || ($parent_node && $parent_node->{type}))) {
+				my $counter_name = $node->{type} || $parent_node->{type};
+				init_counter($counter_name,0) if ! $set_counter_names{$counter_name};
+			}
+		}
 	}
-	
-  for my $key ( keys %$node ) {
+
+  for my $key ( sort keys %$node ) {
 		my $value = $node->{$key};
 		if ( ref $value eq 'HASH' ) {
-			repopulateconfiguration($value,$key);
+			repopulateconfiguration($value,$key,$node,$curr,$do_counter);
 		} elsif ( ref $value eq 'ARRAY' ) {
-			for my $item (@$value) {
+			for my $i (0..@$value-1) {
+				my $item = $value->[$i];
+				#printinfo($i,$key,$item);
 				if ( ref $item ) {
-					repopulateconfiguration($item,$key);
+					repopulateconfiguration($item,$key,$node,$curr,$do_counter);
+				} else {
+					my $new_value     = parse_field($item,$key,$parent_node_name,$node,$curr);
+					$node->{$key}[$i] = $new_value;
 				}
+				#printinfo($i,$key,$node->{$key}[$i]);
 			}
 		} else {
-			if ($key =~ /\s+/) {
-	      fatal_error("configuration","multi_word_key",$key); 
-      }
-			my $new_value = parse_field($value,$key,$parent_node_name,$node);
-      $node->{$key} = $new_value;
+			# excluding the counter key is required because blocks like
+			# <pairwise /hs/ /hs/ >
+			# will trigger the multi_word_key error
+			if ($key =~ /\s+/ && $parent_node_name ne "counter") { 
+	      fatal_error("configuration","multi_word_key",$key);
+      } else {
+				my $new_value = parse_field($value,$key,$parent_node_name,$node,$curr);
+				$node->{$key} = $new_value;
+			}
     }
   }
-	
-  %set_counter_names = set_counters(
-																		$node,
-																		{
-																		 param=>"post_increment_counter", fn=>\&increment_counter },
-																		{
-																		 param=>"post_set_counter",       fn=>\&set_counter       },
-																	 );
-	
+
+	if($do_counter) {
+		%set_counter_names = set_counters(
+																			$node,
+																			$parent_node_name,
+																			$parent_node,
+																			{ param=>"post_increment_counter", fn=>\&increment_counter },
+																			{ param=>"post_set_counter",       fn=>\&set_counter       },
+																		 );
+
   # default post increment counter
-	if (ref $node && defined $parent_node_name) {
-		increment_counter($parent_node_name,1) if ! $set_counter_names{$parent_node_name};
+		if (ref $node && defined $parent_node_name) {
+			my $default_increment = defined $COUNTER->{$parent_node_name}{increment} ? $COUNTER->{$parent_node_name}{increment} : 1;
+			if($parent_node_name eq "plot" &&
+				 ($node->{type} || ($parent_node && $parent_node->{type}))) {
+				my $counter_name = $node->{type} || $parent_node->{type};
+				increment_counter($counter_name,$default_increment) if ! $set_counter_names{$counter_name};
+			}
+			increment_counter($parent_node_name,$default_increment) if ! $set_counter_names{$parent_node_name};
+		}
 	}
-	
 }
 
 sub parse_field {
 
-	my ($str,$key,$parent_node_name,$node) = @_;
+	my ($str,$key,$parent_node_name,$node,$curr) = @_;
 	my $delim    = "__";
-	
-	# replace counters 
+
+	if(! defined $str) {
+		printdumper($node); 
+		fatal_error("configuration","undefined_string",$key,$parent_node_name || "_root");
+	}
+
+	# replace counters
 	# counter(NAME)
 	while ( $str =~ /(counter\(\s*(.+?)\s*\))/g ) {
 		my ($template,$counter) = ($1,$2);
 		if (defined $template && defined $counter) {
-			my $new_template = get_counter($counter);
-			printdebug_group("counter","fetch",$template,$counter,$new_template);
-			$str =~ s/\Q$template\E/$new_template/g;
+		    my $new_template = get_counter($counter);
+		    printdebug_group("counter","fetch",$template,$counter,$new_template);
+		    $str =~ s/\Q$template\E/$new_template/g;
 		}
 	}
+
 	# replace configuration field
 	# conf(LEAF,LEAF,...)
-	while ( $str =~ /(conf\(\s*(.+?)\s*\))/g ) {
-		my ($template,$leaf) = ($1,$2);
-		if (defined $template && defined $leaf) {
-			my @leaf         = split(/\s*,\s*/,$leaf);
-			my $new_template;
-			if (@leaf == 2 && $leaf[0] eq ".") {
-				$new_template = $node->{$leaf[1]};
-			} else {
-				$new_template = fetch_conf(@leaf);
-			}
-			printdebug_group("conf","fetch",$template,join(",",@leaf),$new_template);
-			$str =~ s/\Q$template\E/$new_template/g;
-		}
-	}
-	
+	$str = parse_conf_fn($str,$node,$curr);
+
 	# this is going to be deprecated
-	while ( $str =~ /$delim([^_].+?)$delim/g ) {
-		my $source = $delim . $1 . $delim;
-		my $target = eval $1;
-		printdebug_group("conf","repopulate","key",$key,"value",$str,"var",$1,"target",$target);
-		$str =~ s/\Q$source\E/$target/g;
-		printdebug_group("conf","repopulate",$key,$str,$target);
+	if( $str =~ /$delim([^_].+?)$delim/g ) {
+		fatal_error("configuration","deprecated","__FIELD__","var(FIELD)");
 	}
 	
-	if ($str =~ /\s*eval\s*\(\s*(.+)\s*\)/ && $parent_node_name ne "rule" && $str !~ /var\s*\(/) {
-		my $fn = $1;
-		$str = eval $fn;
-		if ($@) {
-			fatal_error("rules","parse_error",$fn,$@);
+	$str = eval_conf($str) if $str !~ /var\s*\(/ && (! defined $parent_node_name || $parent_node_name ne "rule");
+
+	#$redo = undef;
+	#while(! defined $redo || $redo) {
+	#	$redo = 0;
+	#	while ($str =~ /\s*eval\s*\(\s*(.+)\s*\)/ && $parent_node_name ne "rule" && $str !~ /var\s*\(/) {
+	#		my $fn = $1;
+	#		printinfo($str,$fn);
+	#		printinfo(Circos::Expression::extract_balanced($fn,"eval","(",")"));
+	#		if($fn =~ /eval\(/) {
+	#			$redo = 1;
+	#			next;
+	#		}
+	#		$str = eval $fn;
+	#		if ($@) {
+	#			fatal_error("rules","parse_error",$fn,$@);
+	#		}
+	#		printdebug_group("conf","repopulateeval",$fn,$str);
+	#	}
+	#}
+
+	return $str;
+}
+
+sub parse_conf_fn {
+	my ($str,$node,$curr) = @_;
+	my $redo;
+	while( ! defined $redo || $redo) {
+		$redo = 0;
+		while ( $str =~ /((opt)?conf\(\s*([^\(\)]+?)\s*\))/g ) {
+	    my ($template,$checkonly,$leaf) = ($1,$2,$3);
+	    if($leaf =~ /conf\(/) {
+				$redo = 1;
+				next;
+	    }
+	    if (defined $template && defined $leaf) {
+				my @leaf  = split(/\s*,\s*/,$leaf);
+				my $new_template;
+				if (@leaf == 2 && $leaf[0] eq ".") {
+					if(defined $curr && exists $curr->{$leaf[1]}) {
+						$new_template = $curr->{$leaf[1]};
+					} else {
+						$new_template = $node->{$leaf[1]};
+					}
+				} else {
+					$new_template = fetch_conf(@leaf);
+				}
+				if(! defined $new_template && ! $checkonly) {
+					fatal_error("configuration","no_such_conf_item",$template,$leaf);
+				}
+				printdebug_group("conf","fetch",$template,join(",",@leaf),$new_template);
+				if(defined $new_template) {
+					if($new_template =~ /(var|eval)\(/) {
+
+					} else {
+						$str =~ s/\Q$template\E/$new_template/g;
+					}
+				} else {
+					$str =~ s/\Q$template\E/undef/g;
+				}
+			}
 		}
-		printdebug_group("conf","repopulateeval",$fn,$str);
 	}
 	return $str;
 }
 
+sub eval_conf {
+	my $str = shift;
+	#printinfo("eval_conf",$str);
+	my ($template,$arg) = Circos::Expression::extract_balanced($str,"eval","(",")");
+	if(defined $arg) {
+		#printinfo("eval_conf template",$template);
+		#printinfo("eval_conf      arg",$arg);
+		my $for_eval;
+		if(Circos::Expression::extract_balanced($arg,"eval","(",")")
+			 ||
+			 Circos::Expression::extract_balanced($arg,"counter","(",")")
+			 ||
+			 Circos::Expression::extract_balanced($arg,"conf","(",")")) {
+			$for_eval = parse_field($arg);
+		} else {
+			$for_eval = $arg;
+		}
+		my $value = eval $for_eval;
+		if ($@) {
+			fatal_error("rules","parse_error",$for_eval,$@);
+		}
+		#printinfo("eval_conf    value",$value);
+		$str =~ s/\Qeval$template\E/$value/;
+		#printinfo("eval_conf     nstr",$str);
+	}
+	if(Circos::Expression::extract_balanced($str,"eval","(",")")) {
+		return eval_conf($str);
+	} else {
+		return $str;
+	}
+}
+
 # -------------------------------------------------------------------
 sub loadconfiguration {
   my ($arg,$return) = @_;
@@ -580,16 +787,16 @@ sub loadconfiguration {
   my $file;
   for my $f ( @possibilities ) { 
     if ( -e $f && -r _ ) {
-      printdebug_group("conf","found conf file",$f);
+      printdebug_group("summary","found conf file",$f);
       $file = $f;
       last;
     }
   }
-  
+
   if ( !$file ) {
     fatal_error("configuration","missing");
   }
-  
+
   my @configpath = (
 										dirname($file),
 										dirname($file)."/etc",
@@ -598,8 +805,7 @@ sub loadconfiguration {
 										"$FindBin::RealBin/..",  
 										$FindBin::RealBin,
 									 );
-	
-	
+
 	my $conf;
 	eval {
 		$conf = Config::General->new(
@@ -608,6 +814,8 @@ sub loadconfiguration {
 																 -AllowMultiOptions => 1,
 																 -LowerCaseNames    => 1,
 																 -IncludeAgain      => 1,
+																 -CComments         => 0,
+																 -NormalizeBlock    => sub { my $x = shift; $x =~ s/\s*$//; $x; },
 																 -ConfigPath        => \@configpath,
 																 -AutoTrue => 1
 																);
@@ -629,32 +837,20 @@ sub loadconfiguration {
 # -------------------------------------------------------------------
 sub validateconfiguration {
 
-	if (0) {
-		for my $parsekey ( keys %CONF ) {
-	    if ( $parsekey =~ /^(__(.+)__)$/ ) {
-				if ( !defined $CONF{$1} ) {
-					fatal_error("configuration","bad_pointer",$1,$2);
-				}
-				my ( $token, $parsevalue ) = ($1,$CONF{$1});
-				for my $key ( keys %CONF ) {
-					$CONF{$key} =~ s/$token/$parsevalue/g;
-				}
-	    }
-		}
-	}
-    
-	if ($CONF{debug} && ! $CONF{debug_group}) {
-		$CONF{debug_group} = "summary,io,timer";
-	}
-    
 	$CONF{chromosomes_display_default} = 1 if ! defined $CONF{chromosomes_display_default};
 	$CONF{chromosomes_units} ||= 1;
 	$CONF{svg_font_scale}    ||= 1;
-    
+
 	if ( ! $CONF{karyotype} ) {
 		fatal_error("configuration","no_karyotype");
 	}
-    
+
+	for my $block (qw(ideogram image colors fonts)) {
+		if ( ! $CONF{$block} ) {
+			fatal_error("configuration","no_block",$block);
+		}
+	}
+
 	$CONF{image}{image_map_name} ||= $CONF{image_map_name};
 	$CONF{image}{image_map_use}  ||= $CONF{image_map_use};
 	$CONF{image}{image_map_file} ||= $CONF{image_map_file};
@@ -662,22 +858,78 @@ sub validateconfiguration {
 	$CONF{image}{"24bit"} = 1;
 	$CONF{image}{png}  = $CONF{png} if exists $CONF{png};
 	$CONF{image}{svg}  = $CONF{svg} if exists $CONF{svg};
-	$CONF{image}{file} = $CONF{outputfile} if $CONF{outputfile};
-	$CONF{image}{dir}  = $CONF{outputdir}  if $CONF{outputdir};
+	if(my $file = $CONF{outputfile} || $CONF{file}) {
+		$CONF{image}{file} = $file;
+	}
+	if(my $dir = $CONF{outputdir} || $CONF{dir}) {
+		$CONF{image}{dir} = $dir;
+	}
 	$CONF{image}{background} = $CONF{background} if $CONF{background};
-    
-	if ( $CONF{image}{angle_offset} > 0 ) {
+
+	while($CONF{image}{angle_offset} > 0 ) {
 		$CONF{image}{angle_offset} -= 360;
 	}
-    
+
 	#
 	# Make sure these fields are initialized
 	#
-    
-	for my $fld ( qw(chromosomes chromosomes_breaks chromosomes_radius) ) {
-		$CONF{ $fld } = $EMPTY_STR if !defined $CONF{ $fld };
+
+	for my $f ( qw(chromosomes chromosomes_breaks chromosomes_radius) ) {
+		$CONF{ $f } = $EMPTY_STR if ! defined $CONF{ $f };
+	}
+
+}
+
+# -------------------------------------------------------------------
+# Counters
+
+# -------------------------------------------------------------------
+sub get_counter {
+	my $counter = shift;
+	confess if ! defined $counter;
+	if (! exists_counter($counter)) {
+		fatal_error("configuration","no_counter",$counter);
+	} else {
+		return $CONF{counter}{$counter};
+	}
+}
+
+# -------------------------------------------------------------------
+sub exists_counter {
+	my $counter = shift;
+	return defined $CONF{counter}{$counter};
+}
+
+# -------------------------------------------------------------------
+sub increment_counter {
+  my ($counter,$value) = @_;
+  init_counter($counter,0);
+	if (defined $value) {
+		$COUNTER->{$counter}{increment}  = $value;
+		$CONF{counter}{$counter}        += $value;
+		printdebug_group("counter","incrementing counter",$counter,$value,"now",get_counter($counter));
+	}
+}
+
+sub set_counter {
+  my ($counter,$value) = @_;
+	if (defined $value) {
+		$CONF{counter}{$counter} = $value;
+		printdebug_group("counter","set counter",$counter,$value,"now",get_counter($counter));
+	}
+}
+
+{
+	my %seen;
+	sub init_counter {
+		my ($counter,$value) = @_;
+		if (! $seen{$counter}++) {
+			if (defined $value) {
+				$CONF{counter}{$counter} = $value;
+				printdebug_group("counter","init counter",$counter,"with value",$value,"new value",get_counter($counter));
+			}
+		}
 	}
-    
 }
 
 1;
