@@ -3,29 +3,13 @@ package Circos::SVG;
 =pod
 
 =head1 NAME
-    
+
 Circos::SVG - utility routines for SVG in Circos
 
 =head1 SYNOPSIS
 
 This module is not meant to be used directly.
 
-=head1 DESCRIPTION
-
-Circos is an application for the generation of publication-quality,
-circularly composited renditions of genomic data and related
-annotations.
-
-Circos is particularly suited for visualizing alignments, conservation
-and intra and inter-chromosomal relationships. However, Circos can be
-used to plot any kind of 2D data in a circular layout - its use is not
-limited to genomics. Circos' use of lines to relate position pairs
-(ribbons add a thickness parameter to each end) is effective to
-display relationships between objects or positions on one or more
-scales.
-
-All documentation is in the form of tutorials at L<http://www.circos.ca>.
-
 =cut
 
 # -------------------------------------------------------------------
@@ -36,7 +20,9 @@ use warnings;
 use base 'Exporter';
 our @EXPORT = qw(
 									style_string
+									attr_string
 									style
+									get_patterns
 							 );
 
 use Carp qw( carp confess croak );
@@ -48,7 +34,7 @@ use lib "$FindBin::RealBin";
 use lib "$FindBin::RealBin/../lib";
 use lib "$FindBin::RealBin/lib";
 
-use Circos::Configuration; 
+use Circos::Configuration;
 use Circos::Colors;
 use Circos::Constants;
 use Circos::Debug;
@@ -60,26 +46,29 @@ use Circos::Text;
 use Circos::Unit;
 use Circos::Image qw(!draw_line);
 
-use Memoize;
+our %patterns;
+
+#use Memoize;
 
 our $default_color      = "black";
 our $default_font_name  = "Arial";
 our $default_font       = "default";
 our $default_font_color = "black";
 
-for my $f ( qw ( ) ) {
-	memoize($f);
-}
+#for my $f ( qw ( ) ) {
+#	memoize($f);
+#}
 
 sub draw_polygon {
 	my %params;
 	if ( fetch_conf("debug_validate") ) {
 		%params = validate(@_,{
-													 polygon          => 1,
-													 thickness        => 0,
-													 color            => 0,
-													 fill_color       => 0,
-													 linecap          => 0,
+													 polygon    => 1,
+													 thickness  => 0,
+													 color      => 0,
+													 fill_color => 0,
+													 linecap    => 0,
+													 attr       => { type => HASHREF|UNDEF, optional => 1 },
 													});
 	} else {
 		%params = @_;
@@ -90,7 +79,10 @@ sub draw_polygon {
 										color     => $params{color},
 										linecap   => $params{linecap},
 										fill_color=> $params{fill_color});
-	my $svg = sprintf(q{<polygon points="%s" style="%s"/>},$pts,$style);
+	my $svg = sprintf(q{<polygon points="%s" style="%s" %s/>},
+										$pts,$style,
+										attr_string($params{attr}));
+	printdebug_group("svg","polygon",$pts,$style);
 	Circos::printsvg($svg);
 }
 
@@ -104,16 +96,20 @@ sub draw_circle {
 													 color             => 0,
 													 stroke_color      => 0,
 													 stroke_thickness  => 0,
+													 attr              => { type => HASHREF|UNDEF, optional => 1 },
 													});
 	} else {
 		%params = @_;
 	}
+	return if ! $params{radius};
 	my $style = style(thickness => $params{stroke_thickness},
 										color     => $params{stroke_color},
 										fill_color=> $params{color});
 	my $fmt = "'%.1f'";
-	my $svg = sprintf(qq{<circle cx=$fmt cy=$fmt r=$fmt style="%s"/>},
-										@{$params{point}},$params{radius},$style);
+	my $svg = sprintf(qq{<circle cx=$fmt cy=$fmt r=$fmt style="%s" %s/>},
+										@{$params{point}},$params{radius},$style,
+										attr_string($params{attr}));
+	printdebug_group("svg","circle",@{$params{points}},$params{radius},$style);
 	Circos::printsvg($svg);
 }
 
@@ -128,6 +124,7 @@ sub draw_line {
 													 color            => { default => fetch_conf("default_color") || $default_color  },
 													 thickness        => { default => 1 },
 													 "stroke-linecap" => { default => "round" },
+													 attr             => { type => HASHREF|UNDEF, optional => 1 },
 													});
 	} else {
 		%params = @_;
@@ -135,21 +132,27 @@ sub draw_line {
 		$params{"stroke-linecap"} ||= "round";
 		$params{thickness}        ||= 1;
 	}
-    
+
 	if (@{$params{points}} != 4) {
 		fatal_error("argument","list_size",current_function(),current_package(),4,int(@{$params{points}}));
 	}
-    
+
 	my $fmt = "'%.1f'";
 	my $style = style(thickness  => $params{thickness},
 										color      => $params{color},
 										fill_color => undef,
 										linecap    => $params{"stroke-linecap"});
-	my $svg = sprintf(qq{<line x1=$fmt y1=$fmt x2=$fmt y2=$fmt style="%s"/>},@{$params{points}},$style);
+	my $svg = sprintf(qq{<line x1=$fmt y1=$fmt x2=$fmt y2=$fmt style="%s" %s/>},
+										@{$params{points}},$style,
+										attr_string($params{attr}));
 	printdebug_group("svg","line",@{$params{points}},$params{color},$params{thickness},$style);
 	Circos::printsvg($svg);
 }
 
+sub get_patterns {
+	return %patterns;
+}
+
 sub draw_text {
 	my %params;
 	my @args = remove_undef_keys(@_);
@@ -160,21 +163,24 @@ sub draw_text {
 												 font      => { default => fetch_conf("default_font") || $default_font },
 												 size      => 1,
 												 color     => { default => fetch_conf("default_font_color") || $default_font_color },
-												 
+
 												 angle     => 1,
 												 radius    => 1,
-												 
+
 												 angle_offset => { default => 0 },
-												 
-												 is_parallel => { default => 0 },
-												 is_rotated  => { default => 1 },
-												 
-												 rotation    => { default => 0 },
-												 
-												 x_offset   => { default => 0 },
-												 y_offset   => { default => 0 },
-												 
-												 guides    => { default => 0 },
+
+												 is_parallel  => { default => 0 },
+												 is_rotated   => { default => 1 },
+
+												 rotation     => { default => 0 },
+
+												 x_offset     => { default => 0 },
+												 y_offset     => { default => 0 },
+
+												 guides       => { default => 0 },
+
+												 attr         => { type => HASHREF|UNDEF, optional => 1 },
+												 mapoptions   => 0,
 												});
 	} else {
 		%params = @args;
@@ -192,7 +198,7 @@ sub draw_text {
 	my $text_angle     = text_angle_2( angle => $params{angle}, 
 																		 is_rotated => $params{is_rotated},
 																		 is_parallel => $params{is_parallel} );
-    
+
 	my $radius_offset  = 0;
 	# adjust radius
 	if ($params{is_parallel} ) {
@@ -218,25 +224,25 @@ sub draw_text {
 	    $anchor = "start";
 		}
 	}
-    
+
 	my ($x,$y) = getxypos( $params{angle}  + $params{angle_offset}, 
 												 $params{radius} + $radius_offset );
-    
+
 	my $svg_text = $params{text};
 	$svg_text =~ s/&/&amp;/g;
-    
+
 	my $style = style(thickness  => $params{thickness},
 										fill_color => $params{color},
 										textanchor => $anchor);
-	
-	my $svg = sprintf( qq{<text x="%.1f" y="%.1f" font-size="%.1fpx" font-family="%s" style="%s" transform="rotate(%.1f,%.1f,%.1f)">%s</text>},
-										 $x,$y,
-										 $CONF{svg_font_scale} * $params{size},
-										 $font_name,
-										 $style,
-										 360 - $text_angle,
-										 $x,$y,
-										 defined $svg_text ? $svg_text : $EMPTY_STR,
+	my $svg = sprintf(qq{<text x="%.1f" y="%.1f" font-size="%.1fpx" font-family="%s" style="%s" transform="rotate(%.1f,%.1f,%.1f)" %s>%s</text>},
+										$x,$y,
+										$CONF{svg_font_scale} * $params{size},
+										$font_name,
+										$style,
+										360 - $text_angle,
+										$x,$y,
+										attr_string($params{attr}),
+										defined $svg_text ? $svg_text : $EMPTY_STR,
 									 );
 	Circos::printsvg($svg);
 }
@@ -250,31 +256,41 @@ sub draw_slice {
 																				 end          => 1,
 																				 chr          => 1,
 																				 radius_from  => 1,
-																				 radius_to    => 1,
+																				 radius_to    => 0,
+																				 radius_to_y0 => 0,
+																				 radius_to_y1 => 0,
 																				 edgecolor    => 1,
 																				 edgestroke   => 1,
 																				 fillcolor    => 0,
+																				 pattern      => 0,
 																				 ideogram     => 0,
-																				 mapoptions   => { type => HASHREF, optional => 1 },
+																				 attr         => { type => HASHREF|UNDEF, optional => 1 },
+																				 svg          => 0,
+																				 mapoptions   => { type => HASHREF|UNDEF, optional => 1 },
 																				 guides       => 0,
-																				 
-																				 start_a => 1,
-																				 end_a   => 1,
+																				 start_a      => 1,
+																				 end_a        => 1,
 																				 angle_orientation => 1,
 																				});
-	
+
 	my %params = @_;
 
 	my ($start_a,$end_a,$angle_orientation) = @params{qw(start_a end_a angle_orientation)};
-	
+
 	my $style = style(thickness  => $params{edgestroke},
 										color      => $params{edgecolor},
 										linecap    => $params{linecap} || "round",
 										fill_color => $params{fillcolor});
-	
+
 	my $svg;
-	
-	if ( $params{radius_from} == $params{radius_to} ) {
+
+
+	#printinfo("svg",$params{pattern});
+
+	# register the pattern;
+
+	if (defined $params{radius_to} &&
+			$params{radius_from} == $params{radius_to} ) {
 		my $end_a_mod = $end_a;
 		if ( abs( $end_a - $start_a ) > 359.99 || $start_a == $end_a ) {
 	    $end_a_mod -= 0.01;
@@ -284,7 +300,7 @@ sub draw_slice {
 		# fill because the slice is 0-width
 		#
 		$svg = sprintf(
-									 qq{<path d="M %.2f,%.2f A%.2f,%.2f %.2f %d,%d %.2f,%.2f" style="%s"/>},
+									 qq{<path d="M %.2f,%.2f A%.2f,%.2f %.2f %d,%d %.2f,%.2f" style="%s" %s/>},
 									 getxypos( $start_a, $params{radius_from} ),
 									 $params{radius_from},
 									 $params{radius_from},
@@ -293,13 +309,15 @@ sub draw_slice {
 									 1,
 									 getxypos( $end_a_mod, $params{radius_from} ),
 									 $style,
+									 attr_string($params{attr}),
 									);
-	} elsif ( $end_a == $start_a ) {
+	} elsif ( defined $params{radius_to} && $end_a == $start_a ) {
 		$svg = sprintf(
-									 qq{<path d="M %.2f,%.2f L %.2f,%.2f " style="%s"/>},
+									 qq{<path d="M %.2f,%.2f L %.2f,%.2f " style="%s" %s/>},
 									 getxypos( $start_a, $params{radius_from} ),
 									 getxypos( $end_a,   $params{radius_to} ),
 									 $style,
+									 attr_string($params{attr}),
 									);
 	} else {
 		my $large_arc = abs( $start_a - $end_a ) > 180;
@@ -308,22 +326,92 @@ sub draw_slice {
 		if ( abs( $end_a - $start_a ) > 359.99 || $start_a == $end_a ) {
 	    $end_a_mod -= 0.01;
 		}
-		$svg = sprintf(
-									 qq{<path d="M%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f L%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f Z" style="%s"/>},
-									 # move to (M)
-									 getxypos( $start_a, $params{radius_from} ),
-									 # elliptical arc (A)
-									 $params{radius_from}, $params{radius_from}, # rx ry
-									 40,					# x axis rotation
-									 $large_arc, $sweep, # large arc flag, sweep flag
-									 getxypos( $end_a_mod, $params{radius_from} ), # end x,y
-									 getxypos( $end_a_mod, $params{radius_to} ),   
-									 $params{radius_to}, $params{radius_to},
-									 0,
-									 $large_arc, !$sweep, 
-									 getxypos( $start_a, $params{radius_to} ),
-									 $style,
-									);
+		if(defined $params{radius_to_y0} && defined $params{radius_to_y1}) {
+			# required for flipped orientation
+			my ($ry0,$ry1) = @params{qw(radius_to_y0 radius_to_y1)};
+			if($angle_orientation =~ /counter/) {
+				($ry0,$ry1) = ($ry1,$ry0);
+			}
+			$svg = sprintf(
+										 qq{<path d="M%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f L%.3f,%.3f L%.3f,%.3f Z" style="%s" %s/>},
+										 # move to (M)
+										 getxypos( $start_a, $params{radius_from} ),
+										 # elliptical arc (A)
+										 $params{radius_from}, $params{radius_from}, # rx ry
+										 40,                 # x axis rotation
+										 $large_arc, $sweep, # large arc flag, sweep flag
+										 getxypos( $end_a_mod, $params{radius_from} ), # end x,y
+										 getxypos( $end_a_mod, $ry1 ),
+										 getxypos( $start_a,   $ry0 ),
+										 #getxypos( $start_a,  $params{radius_to} ),
+										 $style,
+										 attr_string($params{attr}),
+										);
+
+			if($params{pattern} && $params{pattern} ne "solid") {
+				my $style = style(thickness  => $params{edgestroke},
+													color      => $params{edgecolor},
+													linecap    => $params{linecap} || "round",
+													pattern    => $params{pattern});
+				$svg .= sprintf(
+											 qq{<path d="M%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f L%.3f,%.3f L%.3f,%.3f Z" style="%s" %s/>},
+											 # move to (M)
+											 getxypos( $start_a, $params{radius_from} ),
+											 # elliptical arc (A)
+											 $params{radius_from}, $params{radius_from}, # rx ry
+											 40,                 # x axis rotation
+											 $large_arc, $sweep, # large arc flag, sweep flag
+											 getxypos( $end_a_mod, $params{radius_from} ), # end x,y
+											 getxypos( $end_a_mod, $ry1 ),
+											 getxypos( $start_a,   $ry0 ),
+											 #getxypos( $start_a,  $params{radius_to} ),
+											 $style,
+											 attr_string($params{attr}),
+											);
+			}
+		} else {
+			$svg = sprintf(
+										 qq{<path d="M%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f L%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f Z" style="%s" %s/>},
+										 # move to (M)
+										 getxypos( $start_a, $params{radius_from} ),
+										 # elliptical arc (A)
+										 $params{radius_from}, $params{radius_from}, # rx ry
+										 40,                 # x axis rotation
+										 $large_arc, $sweep, # large arc flag, sweep flag
+										 getxypos( $end_a_mod, $params{radius_from} ), # end x,y
+										 getxypos( $end_a_mod, $params{radius_to} ),
+										 $params{radius_to}, $params{radius_to},
+										 0,
+										 $large_arc, !$sweep, 
+										 getxypos( $start_a, $params{radius_to} ),
+										 $style,
+										 attr_string($params{attr}),
+										);
+			if($params{pattern} && $params{pattern} ne "solid") {
+				my $style = style(thickness  => $params{edgestroke},
+													color      => $params{edgecolor},
+													linecap    => $params{linecap} || "round",
+													pattern    => $params{pattern});
+				$svg .= sprintf(
+											 qq{<path d="M%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f L%.3f,%.3f A%.3f,%.3f %.3f %d,%d %.3f,%.3f Z" style="%s" %s/>},
+											 # move to (M)
+											 getxypos( $start_a, $params{radius_from} ),
+											 # elliptical arc (A)
+											 $params{radius_from}, $params{radius_from}, # rx ry
+											 40,                 # x axis rotation
+											 $large_arc, $sweep, # large arc flag, sweep flag
+											 getxypos( $end_a_mod, $params{radius_from} ), # end x,y
+											 getxypos( $end_a_mod, $params{radius_to} ),
+											 $params{radius_to}, $params{radius_to},
+											 0,
+											 $large_arc, !$sweep, 
+											 getxypos( $start_a, $params{radius_to} ),
+											 $style,
+											 attr_string($params{attr}),
+											);
+				
+			}
+		}
 	}
 	Circos::printsvg($svg);
 }
@@ -338,9 +426,23 @@ sub style_string {
 	return $style;
 }
 
+################################################################
+# Given a hash, generate an attribute list string
+#
+# key1="value1" key2="value2" key3="value3" ...
+sub attr_string {
+	my $hash = shift;
+	return "" if ref $hash ne "HASH";
+	my @list = ();
+	for my $attr (keys %$hash) {
+		push @list, sprintf(qq{%s="%s"},$attr,$hash->{$attr});
+	}
+	return join(" ",@list);
+}
+
 ################################################################
 # Generate style for
-# 
+#
 # thickness, color, linecap, fill_color, text-anchor
 sub style {
 	my %params = @_;
@@ -359,7 +461,9 @@ sub style {
 	if (my $lc = $params{linecap}) {
 		push @style, sprintf("stroke-linecap:%s",$lc);
 	}
-	if (my $fc = $params{fill_color}) {
+	if (my $pattern = $params{pattern}) {
+		push @style, sprintf("fill:url(#%s)",$pattern);
+	} elsif (my $fc = $params{fill_color}) {
 		push @style, sprintf("fill:rgb(%d,%d,%d)", rgb_color($fc));
 		if ( (my $op = rgb_color_opacity( $params{fill_color} )) < 1 ) {
 			push @style, sprintf("fill-opacity:%.2f",$op);
@@ -373,8 +477,8 @@ sub style {
 sub tag {
 	my $tag = shift;
 	my $tag_data = {
-									xml=>     q|<?xml version="1.0" encoding="utf-8" standalone="no"?>|,
-									doctype=> q|<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">|,
+									xml     => q|<?xml version="1.0" encoding="utf-8" standalone="no"?>|,
+									doctype => q|<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">|,
 								 };
 	if(! $tag_data->{$tag}) {
 		fatal_error("svg","no_such_tag",$tag);
@@ -384,3 +488,33 @@ sub tag {
 }
 
 1;
+
+=pod
+
+=head1 DESCRIPTION
+
+Circos generates circular data visualizations.
+
+Circos is particularly suited for visualizing alignments, conservation
+and intra and inter-chromosomal relationships. However, its use is not
+limited to genomics. Circos can be used to plot any kind of 2D data in
+a circular layout.
+
+Scatter plots, line plots and histograms encode annotation which
+associates a value with a position.
+
+Heatmaps encode values associated with a position with a color
+scheme. Brewer palettes are useful here.
+
+Tiles encode coverage elements like sequence reads.
+
+Connector tracks join two radial and angular positions. These are
+useful to relate positions on two tracks.
+
+Text tracks place position labels.
+
+Lines, curves and ribbons encode relationships between positions.
+
+All documentation is in the form of tutorials at L<http://www.circos.ca>.
+
+=cut
